package tesiProject;

import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;
import java.net.DatagramPacket;

public class Server{
	private final int serverPort = 9999;
	private Database database;
	private DatagramSocket ds;
	private Map<String,ClientStructure> clients = new HashMap<>();
	private Map<String,Integer> access = new HashMap<>();
	private int nRequest = 0, limitRequest = 100;
	private long tStart;

	public Server() {
		try {
			ds = new DatagramSocket(serverPort);
			database = new Database();
			//method = getCPUMethod();
			Puzzle.generatePuzzleKey();
			Puzzle.setCPUMethod();
			tStart = System.currentTimeMillis();
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void startServer() {
		try {
			DatagramPacket dp = new DatagramPacket(new byte[1024], 1024);
			
			while(true) {
				ds.receive(dp); //new packet received
				InetAddress clientAddress = dp.getAddress();
				int remotePort = dp.getPort();
				byte[] receivedData = dp.getData();
				String clientID = clientAddress+":"+remotePort;
				
				if(!clients.containsKey(clientID)) {
					if(!access.containsKey(clientID))
						access.put(clientID, 1);
					else
						access.put(clientID, access.get(clientID)+1);
					int nBitPuzzle = generateNBitPuzzle(access.get(clientID));
					ClientStructure cs = new ClientStructure(receivedData, clientAddress, remotePort, nBitPuzzle);
					clients.put(clientID, cs);
					cs.getPuzzle().sendPuzzle(ds, clientAddress, remotePort);
//					System.out.println("Puzzle solution: "+Functions.bytesToHex(cs.getPuzzle().getH()));
//					System.out.println("Access: "+access);
				}else {
					ClientStructure client = clients.get(clientID);
					if(!client.isConnected()) {
						Puzzle p = client.getPuzzle();
						if(p.getNBitPuzzle() > 0) { //puzzle resolution required
							if(p.checkPuzzle(receivedData)) { //if correct resolution
								client.setConnection(true);
								sendString("OK", clientAddress, remotePort);
							}else { //puzzle not resolved
								sendString("NO", clientAddress, remotePort);
							}
						}else { //no puzzle required
							handleResourceRequest(new String(receivedData, dp.getOffset(), dp.getLength()), clientAddress, remotePort);
							clients.remove(clientID);
						}
					}else {
						handleResourceRequest(new String(receivedData, dp.getOffset(), dp.getLength()), clientAddress, remotePort);
						clients.remove(clientID); //protocol finished
					}
				}
			}
		}catch(Exception e) {
			e.printStackTrace();
		}finally {
			ds.close();
		}
	}
	
	private int generateNBitPuzzle(int nAccess) {
		long tNow = System.currentTimeMillis();
		int nBit = (int)Math.round(((double)nAccess/limitRequest)*20);
//		System.out.println("nAccess/limitRequest vale "+nAccess+"/"+limitRequest+"="+(((double)nAccess/limitRequest)));
		if(nBit == 0)
			return -1;
		return nBit;
//		return -1;
	}
	
	private void handleResourceRequest(String request, InetAddress clientAddress, int remotePort) {
		try {
			String responseRequest = database.getItem(request);
			sendString(responseRequest, clientAddress, remotePort);
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void sendString(String confirm, InetAddress clientAddress, int remotePort) {
		try {
			DatagramPacket dp = new DatagramPacket(confirm.getBytes("UTF-8"), confirm.length(),
					clientAddress, remotePort);			
			
			ds.send(dp);
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
}
